@@ -1,5 +1,5 @@
 #-------------------------------------------------------------------------------
-# Copyright (C) 2006-2019 British Crown (Met Office) & Contributors.
+# Copyright (C) 2006-2021 British Crown (Met Office) & Contributors.
 #
 # This file is part of FCM, tools for managing and building source code.
 #
@@ -130,50 +130,36 @@ sub add_trac_environment {
             $project->get_trac_live_path(),
         ));
     }
-    my @repository_arguments = (q{}, q{});
-    if (-d $project->get_svn_live_path()) {
-        @repository_arguments = (q{svn}, $project->get_svn_live_path());
-    }
     my $RUN = sub{$RUNNER->run(@_)};
     my $TRAC_ADMIN = sub {
         my ($log, @args) = @_;
         my @command = (q{trac-admin}, $project->get_trac_live_path(), @args);
         $RUN->($log, sub {!system(@command)});
     };
-    my $TRAC_ADMIN_CONT = sub {
-        my ($log, @args) = @_;
-        my @command = (q{trac-admin}, $project->get_trac_live_path(), @args);
-        $RUNNER->run_continue($log, sub {!system(@command)});
-    };
     $TRAC_ADMIN->(
         "initialising Trac environment",
         q{initenv},
         $project_name,
         q{sqlite:db/trac.db},
-        @repository_arguments,
         q{--inherit=../../trac.ini},
     );
     my $group = $CONFIG->get_trac_group();
     if ($group) {
         _chgrp_and_chmod($project->get_trac_live_path(), $group);
     }
-    # Note: For some reason, the commands to remove example components,
-    # versions, milestones, priorities fail using the "pip install trac" version
-    # on Travis CI. It is safe to allow the logic to continue after a failure
-    # here as they are really unimportant and can easily be configured later.
     for my $item (qw{component1 component2}) {
-        $TRAC_ADMIN_CONT->(
-            "removing example component $item", q{component remove}, $item,
+        $TRAC_ADMIN->(
+            "removing example component $item", qw{component remove}, $item,
         );
     }
     for my $item (qw{1.0 2.0}) {
-        $TRAC_ADMIN_CONT->(
-            "removing example version $item", q{version remove}, $item,
+        $TRAC_ADMIN->(
+            "removing example version $item", qw{version remove}, $item,
         );
     }
     for my $item (qw{milestone1 milestone2 milestone3 milestone4}) {
-        $TRAC_ADMIN_CONT->(
-            "removing example milestone $item", q{milestone remove}, $item,
+        $TRAC_ADMIN->(
+            "removing example milestone $item", qw{milestone remove}, $item,
         );
     }
     for my $item (
@@ -182,10 +168,17 @@ sub add_trac_environment {
         ['blocker'  => 'critical'],
     ) {
         my ($old, $new) = @{$item};
-        $TRAC_ADMIN_CONT->(
+        $TRAC_ADMIN->(
             "changing priority $old to $new", qw{priority change}, $old, $new,
         );
     }
+    if (-d $project->get_svn_live_path()) {
+        $TRAC_ADMIN->(
+            "adding repository",
+            qw{repository add (default)},
+            $project->get_svn_live_path(),
+        );
+    }
     $TRAC_ADMIN->(
         "adding admin permission", qw{permission add admin TRAC_ADMIN},
     );
@@ -1415,6 +1408,6 @@ L<FCM::Admin::User|FCM::Admin::User>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2006-2019 British Crown (Met Office) & Contributors..
+Copyright (C) 2006-2021 British Crown (Met Office) & Contributors.
 
 =cut
